"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.doCommitPushPublish = void 0;
const devopsCore_1 = require("@thatdevops/core/devopsCore");
const getPkg_1 = __importDefault(require("@thatdevops/core/pkgman/getPkg"));
const getProjectPkgs_1 = __importDefault(require("@thatdevops/core/pkgman/getProjectPkgs"));
const writePkg_1 = __importDefault(require("@thatdevops/core/pkgman/writePkg"));
const Logger_1 = __importDefault(require("@thats/utils/log/Logger"));
const chunk_1 = __importDefault(require("lodash/chunk"));
const luxon_1 = require("luxon");
const path_1 = __importDefault(require("path"));
const BumpType_1 = __importDefault(require("./BumpType"));
const doCalcNewVersions_1 = __importDefault(require("./doCalcNewVersions"));
const doCommit_1 = __importDefault(require("./doCommit"));
const doCommitPush_1 = __importDefault(require("./doCommitPush"));
const doFindChangedPackages_1 = __importDefault(require("./doFindChangedPackages"));
const doFindEffectedPackages_1 = __importDefault(require("./doFindEffectedPackages"));
const doUpdateDependencies_1 = __importDefault(require("./doUpdateDependencies"));
const doYarnPublish_1 = __importDefault(require("./doYarnPublish"));
function doCommitPushPublish(props) {
    return __awaiter(this, void 0, void 0, function* () {
        const { rootPath, parallel = 0, deployable } = (0, devopsCore_1.getDevOpsCore)();
        let { force = false, commitMessage, bumpType = BumpType_1.default.patch, skipcd = false } = props !== null && props !== void 0 ? props : {};
        if (!commitMessage)
            commitMessage = `Publish ${luxon_1.DateTime.now().toISODate()}`;
        const { publishable } = (0, devopsCore_1.getDevOpsCore)();
        if (!publishable) {
            Logger_1.default.error('Publishing not allowed (set the dev ops "publishable" flag');
            return;
        }
        yield (0, doCommit_1.default)(`Pre ${commitMessage}`, deployable);
        Logger_1.default.info('- Finding changed packages');
        let changedPkgs = force ? yield (0, getProjectPkgs_1.default)() : yield (0, doFindChangedPackages_1.default)();
        if (!changedPkgs.length) {
            Logger_1.default.shout('No changes to publish');
            return;
        }
        const effectedPkgs = yield (0, doFindEffectedPackages_1.default)(changedPkgs);
        const newVersions = (0, doCalcNewVersions_1.default)(effectedPkgs, bumpType);
        Logger_1.default.info('- Updating related dependencies');
        yield (0, doUpdateDependencies_1.default)(effectedPkgs, newVersions);
        Logger_1.default.info('- Publishing effected packages');
        if (!!parallel) {
            const chunks = (0, chunk_1.default)(effectedPkgs, parallel);
            for (const chunki of chunks) {
                yield Promise.all(chunki.map(c => (0, doYarnPublish_1.default)(c, newVersions[c.name])));
            }
        }
        else {
            for (const pkg of effectedPkgs) {
                yield (0, doYarnPublish_1.default)(pkg, newVersions[pkg.name]);
            }
        }
        Logger_1.default.info('- Updating DevOps package');
        const devopsPkg = (0, getPkg_1.default)(path_1.default.join(rootPath, 'devops'));
        yield (0, doUpdateDependencies_1.default)([devopsPkg], newVersions);
        yield (0, doCommitPush_1.default)(commitMessage, skipcd);
        Logger_1.default.info('- Setting publish dates');
        const rootPkg = (0, getPkg_1.default)(rootPath);
        rootPkg.json.devops = rootPkg.json.devops || {};
        rootPkg.json.devops.publishdates = rootPkg.json.devops.publishdates || {};
        const now = luxon_1.DateTime.now().plus({ minute: 1 }).toISO();
        for (const pkg of effectedPkgs) {
            rootPkg.json.devops.publishdates[pkg.json.name] = now;
        }
        (0, writePkg_1.default)(rootPkg);
    });
}
exports.doCommitPushPublish = doCommitPushPublish;
exports.default = doCommitPushPublish;
//# sourceMappingURL=doCommitPushPublish.js.map